{ TAMCustomAboutDialog - version 1.1 by AM Software copyright  1997 Alexander Meeder

  This is a simple to use component with a few properties. It is an exact copy of the build-in
  'About-dialog' (Windows API 'ShellAbout) except that it's now fully configurable.

  Properties:
  * AppName: the name of the application, empty by default, 'Application.Title' when empty at runtime
  * Caption: then caption of the dialog, empty by default, About + 'Application.Title' when empty at
             runtime
  * Copyright: default value 'Copyright  199.' where . = current year
  * ExtraInfo: default empty, usefull for a copyright notice or the name of the programmer (max. 2 lines)
  * Hint: default empty, when 'ShowHint' and not empty then this is the hint for the Icon
  * Icon: default empty, when empty at runtim 'Application.Icon'
  * PhysicalMem: default 'Physical Memory Available to Windows:'
  * Registration: default 'This product is licensed to:'
  * ShowHint: default false
  * SystemRes: default 'System Resources'
  * TargetOS: default 'Windows 95'

  Events:
  * OnExecute: just before the About-dialog appears this event occurs
  * OnExecuted: just after the About-dialog is released this event occurs

  You can also test the dialog at runtime by right-clicking the component and choose
  'Test AM Custom About Dialog'

  Version 1.0: First release
  Version 1.1: Second release, repared a problem when using Windows NT in combination with the
               GetFreeRes-function. There is no such function build in Windows NT so I have
               to test for the platform...

  Have a question, suggestion? Mail me at: ameeder@dds.nl
}

unit AMCustAD;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Registry, DsgnIntf;

type
  TAMCustomAboutDialog = class(TComponent)
  private
    FAppName: string;
    FCaption: TCaption;
    FCopyright: string;
    FExtraInfo: string;
    FHint: string;
    FIcon: TIcon;
    FPhysicalMem: string;
    FRegistration: string;
    FShowHint: boolean;
    FSystemRes: string;
    FTargetOS: string;

    FOnExecute: TNotifyEvent;
    FOnExecuted: TNotifyEvent;

    procedure SetIcon(value: TIcon);
    procedure SetHint(value: string);
  protected

  public
    function Execute: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AppName: string read FAppName write FAppName;
    property Caption: TCaption read FCaption write FCaption;
    property Copyright: string read FCopyright write FCopyright;
    property ExtraInfo: string read FExtraInfo write FExtraInfo;
    property Hint: string read FHint write SetHint;
    property Icon: TIcon read FIcon write SetIcon;
    property PhysicalMem: string read FPhysicalMem write FPhysicalMem;
    property Registration: string read FRegistration write FRegistration;
    property ShowHint: boolean read FShowHint write FShowHint default False;
    property SystemRes: string read FSystemRes write FSystemRes;
    property TargetOS: string read FTargetOS write FTargetOS;
    property OnExecute: TNotifyEvent read FOnExecute write FOnExecute;
    property OnExecuted: TNotifyEvent read FOnExecuted write FOnExecuted;
  end;

  TAMCustomAboutDialogEditor = class(TDefaultEditor)
  public
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: integer; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

  TfrmAboutDlg = class(TForm)
    imgIcon: TImage;
    btnOk: TButton;
    lblApplicationName: TLabel;
    lblTargetOS: TLabel;
    lblCopyright: TLabel;
    lblExtraInformation: TLabel;
    lblRegistrationMessage: TLabel;
    lblUserName: TLabel;
    lblCompanyName: TLabel;
    Bevel1: TBevel;
    lblPhysicalMemory: TLabel;
    lblSystemResources: TLabel;
    lblPhMem: TLabel;
    lblSysRes: TLabel;
    procedure FormCreate(Sender: TObject);
  private
    Reg: TRegistry;
    MemStat: TMemoryStatus;
  public

  end;

procedure Register;

implementation

{$R *.DFM}

// Beginning of several declarations and functions to implement the GetFreeSystemResource

const
  Gfsr_SystemResources = 0;
  Gfsr_GdiResources = 1;
  Gfsr_UserResources = 2;

function LoadLibrary16(LibraryName: PChar): THandle; stdcall; external kernel32 index 35;
procedure FreeLibrary16(HInstance: THandle); stdcall; external kernel32 index 36;
function GetProcAddress16(Hinstance: THandle; ProcName: PChar): Pointer; stdcall; external kernel32 index 37;
procedure QT_Thunk; cdecl; external kernel32 name 'QT_Thunk';

var
  hInst16: THandle;
  GFSR: Pointer;
  frmAboutDlg: TfrmAboutDlg;

{$StackFrames On}

function IsWindows95: boolean;
var
  OS: TOSVersionInfo;
begin
  OS.dwOSVersionInfoSize := sizeof(TOSVERSIONINFO);
  GetVersionEx(OS);
  Result := (OS.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS);
end;

function GetFreeSystemResources(SysResource: Word): Word;
var
  ThunkTrash: array[0..$20] of Word;
begin
  Result := 0;
  if IsWindows95 then
  begin
    ThunkTrash[0] := hInst16;

    hInst16 := LoadLibrary16('user.exe');
    if hInst16 < 32 then raise Exception.Create('Cannot load USER.EXE');

    FreeLibrary16(hInst16);

    GFSR := GetProcAddress16(hInst16, 'GetFreeSystemResources');
    if GFSR = nil then raise Exception.Create('Cannot get address of GetFreeSystemResources');

    asm
      push SysResource
      mov edx, GFSR
      call QT_Thunk
      mov Result, ax
    end;
  end;
end;

function GetFreeRes: string;
begin
  Result := IntToStr(GetFreeSystemResources(Gfsr_SystemResources)) + '% Free';
end;

// Beginning of implementation of then ComponentEditor interface

function TAMCustomAboutDialogEditor.GetVerbCount: integer;
begin
  Result := 1;
end;

function TAMCustomAboutDialogEditor.GetVerb(Index: integer): string;
begin
  Result := '&Test AM Custom About-dialog';
end;

procedure TAMCustomAboutDialogEditor.ExecuteVerb;
begin
  (Component as TAMCustomAboutDialog).Execute;
end;

// Beginning of implementation of then Component interface

procedure TAMCustomAboutDialog.SetIcon(value: TIcon);
begin
  FIcon.Assign(value);
end;

procedure TAMCustomAboutDialog.SetHint(value: string);
begin
  if value <> FHint then
  begin
    FHint := value;
    if FHint <> '' then ShowHint := True else ShowHint := False;
  end;
end;

function TAMCustomAboutDialog.Execute: boolean;
begin
  if Assigned(FOnExecute) then FOnExecute(Self);
  frmAboutDlg := TfrmAboutDlg.Create(Application);
  try
    with frmAboutDlg do
    begin
      if AppName = '' then
        lblApplicationName.Caption := Application.Title
      else
        lblApplicationName.Caption := AppName;
      if Self.Caption = '' then
        Caption := 'About ' + Application.Title
      else
        Caption := Self.Caption;
      lblCopyright.Caption := Copyright;
      lblExtraInformation.Caption := ExtraInfo;
      imgIcon.Hint := Self.Hint;
      imgIcon.ShowHint := Self.ShowHint;
      if Self.Icon.Empty then
        imgIcon.Picture.Icon.Assign(Application.Icon)
      else
        imgIcon.Picture.Icon.Assign(Self.Icon);
      lblPhysicalMemory.Caption := PhysicalMem;
      lblRegistrationMessage.Caption := Registration;
      lblSystemResources.Caption := SystemRes;
      lblTargetOS.Caption := TargetOS;
      Result := frmAboutDlg.ShowModal = mrOk;
    end;
  finally
    frmAboutDlg.Free;
  end;
  if Assigned(FOnExecuted) then FOnExecuted(Self);
end;

constructor TAMCustomAboutDialog.Create(AOwner: TComponent);
var
  Year, Dummy: word;
begin
  inherited Create(AOwner);
  FIcon := TIcon.Create;
  FShowHint := False;
  DecodeDate(Now, Year, Dummy, Dummy);
  Copyright := 'Copyright  ' + IntToStr(Year) + ' ...';
  PhysicalMem := 'Physical Memory Available to Windows:';
  Registration := 'This product is licensed to:';
  SystemRes := 'System Resources:';
  TargetOS := 'Windows 95';
end;

destructor TAMCustomAboutDialog.Destroy;
begin
  FIcon.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Dialogs', [TAMCustomAboutDialog]);
  RegisterComponentEditor(TAMCustomAboutDialog, TAMCustomAboutDialogEditor);
end;

// Beginning of TfrmAboutDlg-source

procedure TfrmAboutDlg.FormCreate(Sender: TObject);
var
  ShowSysRes: boolean;
begin
  ShowSysRes := IsWindows95;

  Reg := TRegistry.Create;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion', True);
      LazyWrite := True;
      lblUserName.Caption := ReadString('RegisteredOwner');
      lblCompanyName.Caption := ReadString('RegisteredOrganization');
    end;
  finally
    Reg.Free;
  end;

  MemStat.dwLength := SizeOf(TMemoryStatus);
  GlobalMemoryStatus(MemStat);
  lblPhMem.Caption := Format('%d KB',[MemStat.dwTotalPhys div 1024]);

  lblSystemResources.Visible := ShowSysRes;
  lblSysRes.Visible := ShowSysRes;
  if ShowSysRes then lblSysRes.Caption := GetFreeRes;
end;

end.
